// JTelnet.CPP
//
// Copyright (c) 1998 John McAleely
//

// NewDocumentL() is called when the document content needs to be reset or 
// re-seeded to default (or "factory") settings. The implementation here 
// simply resets the model.
// In this example, NewDocumentL() is called when a new document
// file is created (typically, in response to the "Create new file" command,
// invoked by selecting the appropriate menu item or pressing the associated
// short-cut key combination). 
//  
// StoreL() and RestoreL() provide the behaviour for storing and restoring
// the document's contents, usually the application's model data. 
// StoreL() and RestoreL() are called both by the application framework and 
// by application defined member functions. StoreL() is called when creating
// or saving a document file; RestoreL() is called when opening a
// document file. 
// Note that document files are dictionary file stores; in particular, note
// that both StoreL() and RestoreL() are passed a reference to the file store
// and the stream dictionary.
//
// NewL() is a static member function responsible for constructing a 
// document object. In this example, construction involves the construction
// of the model component
//
// DoNewFileL() is called by the Ui in response to the "Create new file"
// command, invoked by selecting the appropriate menu item or pressing the
// associated short-cut key combination. The function creates a new document
// file store, resets the document content, closes the existing document
// file store and tells the application process that the new file is now 
// the main document file.


#include <coemain.h>

#include <eikenv.h>
#include <eikgted.h>
#include <eikcapca.h>
#include <eikmfne.h>
#include <eikproc.h>
#include <eikdialg.hrh>
#include <s32file.h>

#include <netdial.h>
//#include <nifvar.h>
#include <in_iface.h>
#include <plpsess.h>

#include "JTelnet.h"
#include "JTelnet.hrh"
#include <JTelnet.rsg>
#include "nvt.h"
#include "helptask.h"
#include "JTAppUI.h"



//
// CJTDocument
//


//
// Constructor
//
CJTDocument::CJTDocument(CEikApplication& aApp)
: CEikDocument(aApp)
, iAddr(NULL)
{
}


//
// destructor
//
CJTDocument::~CJTDocument() {
    delete iAddr;
}


//
// NewL
//
CJTDocument* CJTDocument::NewL(CEikApplication& aApp) {

    CJTDocument* self=new(ELeave) CJTDocument(aApp);
    CleanupStack::PushL(self);
    
    self->CreateModelL();
    
    CleanupStack::Pop();
    return self;
}


//
// ResetModelL
//
void CJTDocument::ResetModelL() {
    delete iAddr;
    CreateModelL();
}


//
// CreateModelL
//
void CJTDocument::CreateModelL() {
    iAddr = CUIInetAddr::NewLC(_L(""), CNVT::EDefaultPort);
    CleanupStack::Pop();
}


//
// CreateAppUiL
//
CEikAppUi* CJTDocument::CreateAppUiL() {
    return(new(ELeave) CJTAppUi);
}


//
// NewDocumentL
//
void CJTDocument::NewDocumentL() {
    ResetModelL();
}


//
// NewDocumentL
//
void CJTDocument::NewDocumentL(const CUIInetAddr& aAddr) {
    NewDocumentL();

    iAddr->CopyL(aAddr);
}


//
// DoNewFileL
//
// Create a new file, and setup the server address
void CJTDocument::DoNewFileL(const TFileName& aFileName, const CUIInetAddr& aAddr) {
    
    // Create a new store (i.e. the file and the store object) 
    CFileStore* store =  CreateFileStoreLC(Process()->FsSession(),aFileName);
    
    // Reset the document back to its factory settings
    NewDocumentL(aAddr);
    
    // Remove store from the cleanup stack
    CleanupStack::Pop();  
    
    // Delete the old store object (closes the file)
    delete(((CEikProcess*)Process())->MainStore());
    
    // Set the new store as the main store.
    ((CEikProcess*)Process())->SetMainStore(store);
    
    // Set the file name of the new store as the main file name
    Process()->SetMainDocFileName(aFileName);
}

void CJTDocument::DoOpenFileL(const TFileName& aFileName)
	{
	  // Open the store and construct the file store object and the stream
	  // dictionary object
   	CFileStore* store = NULL;
	CStreamDictionary* dic = CApaProcess::ReadRootStreamLC(Process()->FsSession(),store,aFileName,EFileShareExclusive|EFileWrite);
	  // Push the store onto the cleanup stack 
	CleanupStack::PushL(store);
	  // Restore content from this store
	RestoreL(*store,*dic);
	  // Can now remove the store from the cleanup stack
	CleanupStack::Pop();
	  // Cleanup the stream dictionary
	CleanupStack::PopAndDestroy();
	   // Delete the old store object (closes the file)
	delete(((CEikProcess*)Process())->MainStore());
	  // Set the new store as the main store.
	((CEikProcess*)Process())->SetMainStore(store);
	  // Set the file name of the new store as the main file name
	Process()->SetMainDocFileName(aFileName);
	}

void CJTDocument::DoSaveToNewFileL(const TFileName& aNewFileName)
	{
	  // Constructs a new store of this name, closes the current store,
	  // and makes the new store the main one.
	((CEikProcess*)Process())->SaveToDirectFileStoreL(this,&aNewFileName);
	SetChanged(EFalse);
	}


TBool CJTDocument::FileNameExists(const TFileName& aFileName) const {
    return (aFileName==Process()->MainDocFileName());
}


//
// StoreL
//
// Perist this document
void CJTDocument::StoreL(CStreamStore& aStore,CStreamDictionary& aStreamDic) const {

    TStreamId id=iAddr->StoreL(aStore);
    aStreamDic.AssignL(KUidJTelnetApp,id);
}


//
// RestoreL
//
// read this document in from persistent store.
void CJTDocument::RestoreL(const CStreamStore& aStore,const CStreamDictionary& aStreamDic) {

    TStreamId id = aStreamDic.At(KUidJTelnetApp);
    CUIInetAddr* file_contents = CUIInetAddr::NewLC(aStore, id);

    ResetModelL();

    iAddr->CopyL(*file_contents);

    CleanupStack::PopAndDestroy();
}



//
// --- CJTelnetApp ---
//


CJTelnetApp::CJTelnetApp() 
: CEikApplication()
, iNif()
, iNIFOpened(EFalse)
, iReEnableLink(EFalse) {
}

CJTelnetApp::~CJTelnetApp() {
    if (iNIFOpened) {
	iNif.Close();
    }
}

// 
// AppDllUid
//
// return our app uid
TUid CJTelnetApp::AppDllUid() const {
    return KUidJTelnetApp;
}


//
// CreateDocumentL
//
// return a new telnet document
CApaDocument* CJTelnetApp::CreateDocumentL() {
    
    return CJTDocument::NewL(*this);
}


//
// IsNetworkOpen
// 
// Open the network layer and ask if it is active.
TBool CJTelnetApp::IsNetworkOpen() {

    TBool res = EFalse;

    if (!iNIFOpened) {
	TInt r = iNif.Open();
	if (r == KErrNone) {
	    iNIFOpened = ETrue;
	}
    }

    if (iNIFOpened) {

	TNifProgress state;
        if (iNif.Progress(state) == KErrNone) {

	    switch (state.iStage) {
	    case EStartingSelection:
	    case EFinishedSelection:
	    case EStartingDialling:
	    case EFinishedDialling:
	    case EStartingLogIn:
	    case EFinishedLogIn:
	    case EStartingHangUp:
	    case EGettingLoginInfo:
	    case EGotLoginInfo:
//	    case EStartingConnect:
//	    case EFinishedConnect:
//	    case EStartingAnswer:
//	    case EAnswered:
                res = ETrue;	// connecting
                break;
	    case KAgentUninitialised:
	    case KAgentUnconnected:
	    case EIfProgressLinkDown:
	    case EFinishedHangUp:
                res = EFalse;	// disconnected
                break;
	    case EIfProgressLinkUp:
	    case EConnectionOpen:
                res = ETrue;	// connected
                break;
	    default:
		//Panic(ErrBadNifState);
		;
            }
	}
    }
    return res;
}


//
// CloseNetwork
//
// Shut the network layer down (for hanging up the phone line)
void CJTelnetApp::CloseNetwork() {

    RNif nif;
    if (nif.Open() == KErrNone) {
	nif.Stop();
	nif.Close();

	ReEnableRemoteLink();
    }
}


//
// Disable the remote link if it's active
//
void CJTelnetApp::DisableRemoteLink() {
    RSemaphore  sem;
    RRemoteLink link;
    
    if(sem.OpenGlobal(REMOTELINK_CONFIG_SEMAPHORE)==KErrNone) {
	// the link is active

	if(link.Open()==KErrNone) {
	    link.Disable();
	    iReEnableLink = ETrue;
	    link.Close();
	}
	sem.Close();
    }
}


//
// Re-enble the remote link at the previous settings 
//
void CJTelnetApp::ReEnableRemoteLink() {
    if (iReEnableLink) {
	RRemoteLink link;
	TInt err = link.Open();
	if (err==KErrNone) {
	    link.Enable(EBps115200, TPtrC(0,0), TPtrC(0,0)); // use previous stored settings
	    link.Close();
	}
    }
}


void CJTelnetApp::SetPrefTInt32L(TInt32 aPref, const TUid& aUid) {
    // Open the "ini" file. Application architecture creates the "ini" file
    // if it does not exist
    CDictionaryStore* iniFile = OpenIniFileLC(iCoeEnv->FsSession());

    // The stream containing the background preference will be associated
    // with a Uid. AssignL() creates the stream if it does not exist
    RDictionaryWriteStream writeStream;
    writeStream.AssignLC(*iniFile,aUid);
    writeStream.WriteInt32L(aPref);
    writeStream.CommitL();
    CleanupStack::PopAndDestroy();

    // Commit changes to the store
    if (iniFile->Commit()!=KErrNone) {
	iniFile->RevertL(); 
    }

    // Destroys the "ini" file store object and closes
    // the "ini" file
    CleanupStack::PopAndDestroy(); // iniFile
}


TInt32 CJTelnetApp::GetPrefTInt32L(const TUid& aUid, TInt32 aDefaultPref) {
    TInt32 pref = aDefaultPref;

    // Open the "ini" file. Application architecture creates the "ini" file
    // if it does not exist
    CDictionaryStore* iniFile = OpenIniFileLC(iCoeEnv->FsSession());

    // The stream containing the background preference will be associated
    // with a Uid.
    // Open the stream associted with the Uid; it will be created
    // if it does not exist.
    RDictionaryReadStream readStream;
    readStream.OpenL(*iniFile, aUid);

    // If the stream has just been created (because the "ini" file has just
    // been created), it will be "empty" and attempting to read from 
    // the stream will leave with a KErrEof.
    // If the stream is not "empty", background now contains the
    // value stored.
    TRAPD(err, pref = readStream.ReadInt32L());
    readStream.Close();

    // Can now delete the "ini" file object (i.e. close the "ini" file)
    CleanupStack::PopAndDestroy();

    // Return the value retrieved from the "ini" file
    if (!err) {
	return pref;
    }

    // Where the "ini" file has just been created, set a default
    // value and return it.
    pref = aDefaultPref;
    SetPrefTInt32L(pref, aUid);
    return pref;
}



//
// --- EXPORTed functions ---
//

EXPORT_C CApaApplication* NewApplication() {
    return (new CJTelnetApp);
}


GLDEF_C TInt E32Dll(TDllReason)
	{
	return(KErrNone);
	}



CUIInetAddr::CUIInetAddr()
: CBase()
, iPort(KInetPortNone)
, iName(NULL)
{
}


void CUIInetAddr::ConstructL(const TDesC& aName, const TUint& aPort) {
    iName = HBufC::NewL(aName.Length());
    iName->Des() = aName;
    iPort = aPort;
}

void CUIInetAddr::ConstructL(const CEikEdwin* aEdwin, const TUint& aPort) {
    iName = aEdwin->GetTextInHBufL();
    iPort = aPort;
}


CUIInetAddr::~CUIInetAddr() {
    delete iName;
}

CUIInetAddr* CUIInetAddr::NewLC(const TDesC& aName, const TUint& aPort) {
    CUIInetAddr* pNew = new (ELeave) CUIInetAddr();
    CleanupStack::PushL(pNew);
    pNew->ConstructL(aName, aPort);
    return pNew;
}


CUIInetAddr* CUIInetAddr::NewLC(const CEikEdwin* aEdwin, const TUint& aPort) {
    CUIInetAddr* pNew = new (ELeave) CUIInetAddr();
    CleanupStack::PushL(pNew);
    pNew->ConstructL(aEdwin, aPort);
    return pNew;
}

CUIInetAddr* CUIInetAddr::NewLC(const CStreamStore& aStore, TStreamId anId) {
    CUIInetAddr* pNew = new (ELeave) CUIInetAddr();
    CleanupStack::PushL(pNew);
    pNew->ConstructL(_L(""), KInetPortNone);
    pNew->RestoreL(aStore, anId);
    return pNew;
}

CUIInetAddr* CUIInetAddr::NewL(const TDesC& aName, const TUint& aPort) {
    CUIInetAddr* pNew = NewLC(aName, aPort);
    CleanupStack::Pop();
    return pNew;
}


//
// CopyL
//
void CUIInetAddr::CopyL(const CUIInetAddr& alhs) {
    if (&alhs != this) {
	iPort = alhs.Port();
	NameL(alhs.Name());
    }
}


//
// NameL
//
void CUIInetAddr::NameL(const TDesC& aName) {
    if (aName.Length() > iName->Des().MaxLength()) {
	iName = iName->ReAllocL(aName.Length());
    }

    iName->Des() = aName;
}

//
// GetNameL
//
// get the name from this edwin...
void CUIInetAddr::GetNameL(const CEikEdwin* aEdwin) {
    if (aEdwin) {

	if (aEdwin->TextLength() > iName->Des().MaxLength()) {
	    iName = iName->ReAllocL(aEdwin->TextLength());
	}
	aEdwin->GetText(iName->Des());
    }
}


TBool CUIInetAddr::IsValidHostL() const {
    return IsValidHostNameL(*iName) && (iPort != KInetPortNone);
}

TBool CUIInetAddr::IsValidHostNameL(CEikEdwin* aEdwin) {
    CUIInetAddr* candidate = CUIInetAddr::NewLC(aEdwin, KInetPortNone);
    TBool res = candidate->IsValidHostL();
    CleanupStack::PopAndDestroy();
    return res;
}


//
// IsValidHostNameL
//
// Validate this buffer as a potential host name.
// should permit IP addresses (x.x.x.x) or names (www.www)
// for now just forbid zero length strings and spaces...
TBool CUIInetAddr::IsValidHostNameL(const TDesC& aName) {
    if (   aName.Length() > 0 
	&& aName.Locate(' ') == KErrNotFound
	) {
	return ETrue;
    }
    else {
	return EFalse;
    }
}


//
// StoreL
//
TStreamId CUIInetAddr::StoreL(CStreamStore& aStore) {

    RStoreWriteStream outstream;
    TStreamId id = outstream.CreateLC(aStore);
    ExternalizeL(outstream);
    outstream.CommitL();
    CleanupStack::PopAndDestroy();
    return id;
}


//
// ExternalizeL
//
void CUIInetAddr::ExternalizeL(RWriteStream& aStream) const {

    aStream.WriteUint32L(STATIC_CAST(TUint32, iPort));

    TInt32 name_length = iName->Length();
    aStream.WriteInt32L(name_length);
    if (name_length) {
	aStream.WriteL(*iName, name_length);
    }
}
	

//
// RestoreL
//
void CUIInetAddr::RestoreL(const CStreamStore& aStore, TStreamId anId) {	
    RStoreReadStream instream;
    instream.OpenLC(aStore,anId);
    InternalizeL(instream);
    CleanupStack::PopAndDestroy();			
}


//
// InternalizeL
// 
void CUIInetAddr::InternalizeL(RReadStream& aStream) {

    iPort = aStream.ReadUint32L();
 
    TInt32 name_length = aStream.ReadInt32L();
    if (name_length) {
	if (name_length > iName->Des().MaxLength()) {
	    iName = iName->ReAllocL(name_length);
	}
	aStream.ReadL(iName->Des(), name_length);
    }
}


CJTNewFile::CJTNewFile( TDes*  aFileName
		      , TBool* aUseNewFile
		      , TInt   aTitleId
		      , CUIInetAddr& aAddr
		      )
: CEikFileSaveAsDialog(aFileName, aUseNewFile, aTitleId)
, iAddr(aAddr)
{
}


TBool CJTNewFile::OkToExitL(TInt aButtonId) {

    iAddr.Port(STATIC_CAST(CEikNumberEditor*, Control(EJTelnetDlgPortNo))->Number());
    iAddr.GetNameL(STATIC_CAST(CEikEdwin*, Control(EJTelnetDlgInetAddr)));

    if (   aButtonId != EEikBidCancel
	&& !iAddr.IsValidHostL()
	) {
	iEikonEnv->InfoMsg(R_JTELNET_BAD_ADDRESS);
	TryChangeFocusToL(EJTelnetDlgInetAddr);
	return EFalse;
    }

    return CEikFileSaveAsDialog::OkToExitL(aButtonId);
}

void CJTNewFile::PreLayoutDynInitL() {

    // insert above the bottom line containing buttons.

    InsertLineL(iLines->Count() - 1, R_JTELNET_INETADDRESS);
    InsertLineL(iLines->Count() - 1, R_JTELNET_INETPORT);
    InsertLineL(iLines->Count() - 1, R_JTELNET_TELNET_PORT_NOTE);

    SetNumberEditorValue(EJTelnetDlgPortNo, iAddr.Port());
    SetEdwinTextL(EJTelnetDlgInetAddr, &iAddr.Name());

    CEikFileSaveAsDialog::PreLayoutDynInitL();
}



CJTServerDetailsDlg::CJTServerDetailsDlg(CUIInetAddr& aAddr, TBool aReadOnly)
: CEikDialog()
, iAddr(aAddr)
, iReadOnly(aReadOnly)
{
}

void CJTServerDetailsDlg::PreLayoutDynInitL() {

    SetNumberEditorValue(EJTelnetDlgPortNo, iAddr.Port());
    SetEdwinTextL(EJTelnetDlgInetAddr, &iAddr.Name());

    SetLineDimmedNow(EJTelnetTerminalType, ETrue);

    if (iReadOnly) {
	SetLineDimmedNow(EJTelnetDlgPortNo, ETrue);
	SetLineDimmedNow(EJTelnetDlgInetAddr, ETrue);
    }
}

TBool CJTServerDetailsDlg::OkToExitL(TInt aButtonId) {

    if (!iReadOnly) {

	iAddr.Port(STATIC_CAST(CEikNumberEditor*, Control(EJTelnetDlgPortNo))->Number());
	iAddr.GetNameL(STATIC_CAST(CEikEdwin*, Control(EJTelnetDlgInetAddr)));

	if (   aButtonId != EEikBidCancel
	    && !iAddr.IsValidHostL()
	    ) {
	    iEikonEnv->InfoMsg(R_JTELNET_BAD_ADDRESS);
	    TryChangeFocusToL(EJTelnetDlgInetAddr);
	    return EFalse;
	}
    }

    return ETrue;   // only permit OK with valid Inetaddr?
}
